#!/usr/local/bin/perl -w

######################### LAST UPDATED: Jan, 2003 #########################
 
# hardOne.pl version 0.1

###########################################################################
#
# This program converts `hard' data as in the SOURCE file into
# Senseval-1 data format. The output is placed in the DESTINATION file.
# The instance IDs in the SOURCE file are also printed in the FILTER 
# file. Each sentence is placed within the <s> and </s> tags.
# The instances are tagged with the sense SENSE.
# It prints the number of instances converted to Senseval-1 and thereby
# Senseval-2 data format. 
# Exactly one blank line is placed between instances.
# Morphological forms of hard which may constitute the head word are 
# assumed to be... 
#
#	hard, harder, hardest, HARD, HARDER and HARDEST
#
# Compounds present in the form hard-scrabble, hard-cover etc are split to
# the form...
# 	hard <compound>scrabble</compound>, hard <compound>cover</compound>
# Note: This splitting is done only to compounds which have the head word
#
# USAGE :  hardOne.pl [OPTIONS] SENSE FILTER DESTINATION SOURCE
#
###########################################################################

###########################################################################
#
# Copyright (C) 2000-2002
# Ted Pedersen, University of Minnesota, Duluth
# tpederse@d.umn.edu
# Mohammad Saif, University of Minnesota, Duluth
# moha0149@d.umn.edu
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
#
###########################################################################

############################ START OF PROGRAM #############################

use Getopt::Long;

# Check if any options specified
GetOptions("help");
if ( defined $opt_help )    { $help = $opt_help; }
else                        { $help = 0; }
# Print help if asked
if ( $help ) {
    &help();
    exit; }

# Check if any options specified
GetOptions("help");
if ( defined $opt_help )    { $help = $opt_help; }
else                        { $help = 0; }
# Print help if asked
if ( $help ) {
    &help();
    exit; }

# Check if command line options have been provided
# If not print usage information
if ( $#ARGV < 3 ) {
    &usage();
    exit; }

$sense = shift();
$filter = shift();
$opfl = shift();
$inpfl = shift();

if(-f $opfl) {
    print "Destination file already exists. Press `y' to overwrite\n";
    $resp = <STDIN>;
    chomp($resp);
    if($resp eq "y" || $resp eq "Y") {
        print "Overwriting the file.\n"; }
    else {
        exit; } }

# Open files
open(FILTER, ">$filter");
open(OP,">$opfl");
open(INP,"<$inpfl");

# BEGIN PROCESSING

# Read lines from the file - line by line
while($ln = <INP>) {
    # Ignore line if blank
    if($ln =~ /^\s*$/) { next; }
    
    # Correct one particular instance of token merging
    if($ln =~ /^sjm-074:/) {
	$ln =~ s/HARDplastic/HARD plastic/; }
    
    # Capture instance id
    if($ln =~ s/^\s*([^:]+:)(.*)$/$1\n<s> $2 <\/s>/) {
	# Increment number of instances
	$instanceNo++;
	$instid = $1;
	print FILTER "$instid\n"; }
    else {
	print "$instanceNo : Line without Instance ID\n";
	exit(); }
       
    $ln =~ s/((hard|HARD)(er|ER|est|EST)?)-([\w-]+)/$1 <compound>$4<\/compound> /g;
	
    # tag the head word line; possible forms line, lines
    if(!($ln =~ s/(\n|[^a-zA-Z])((hard|HARD)(er|ER|est|EST)?)([^a-zA-Z]|$)/$1 <tag \"$sense\">$2<\/> $5/)) {
    	print "$instanceNo : Line without head word\n";
    	exit(); }

    print OP "$ln\n"; }

print "$inpfl converted. $instanceNo instances in all.\n";

#################### Subroutine to print help message #####################

sub help {
    print "Usage: hardOne.pl [OPTIONS] SENSE FILTER DESTINATION SOURCE\n";
    print "This program converts hard data as in the SOURCE file into Senseval-1 data format. The output is placed in the DESTINATION file. The instance IDs in the SOURCE file are also printed in the FILTER file. The instances are tagged with SENSE\n";
    print "OPTIONS:\n\n";
    print "  --help             Prints the help message.\n\n"; }

sub usage {
    print "Usage: hardOne.pl [OPTIONS] SENSE FILTER DESTINATION SOURCE\n"; 
    AskHelp(); }
# Message describing way to get help
sub AskHelp {
    print "Type hardOne.pl --help for help.\n"; }

############################### END PROGRAM ###############################
